/*
 * Configuration.cs
 * $Id: Configuration.cs,v 1.7 2004/03/31 08:04:56 Mayuki Sawatari Exp $
 *
 * Copyright (C) 2003 Mayuki Sawatari, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE MISUZILLA.ORG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
 
using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;
using System.Drawing;
using System.Resources;
using System.ComponentModel;
using System.Collections;
using System.Collections.Specialized;
using System.Runtime.InteropServices;

using Misuzilla.Utilities;
using Misuzilla.Windows.Forms;
using Misuzilla.Net.GDIPlus.GDIPStrings;

namespace
Misuzilla.Net.GDIPlus.GDIPStrings
{
	internal class
	Configuration
	{
		private GDIPStrings _gdi = null;
		
		private Font _textFont;
		private Boolean _useInputForm = false;
		private Boolean _shadow       = false;
		private Boolean _useFullFadeOut = true;
		private Boolean _revertEnter  = false;
		private Double _FadeWait = 25;
		private Byte _FadeRatio  = 25;
		private Byte _logFormOpacity    = 255;
		private Byte _logFormOpacityMin = 77;
		private Byte _inputFormOpacity  = 255;
		private Int32 _stringHeight = 0;
		private Int32 _lines    = 10;
		private Int32 _width    = 0;
		private Int32 _left     = 0;
		private Int32 _top      = 0;
		private Int32 _inputFormTop     = 0;
		private Int32 _inputFormLeft    = 0;
		private Int32 _tonedownInterval = 5000;
		
		private Regex _ignore = null;
		private Regex _keyword = null;
		private String _browserPath = "";
		
		private HotkeyModifiers _hotkeyMod         = HotkeyModifiers.None;
		private HotkeyModifiers _hotkeyModFade     = HotkeyModifiers.None;
		private HotkeyModifiers _hotkeyModInputWnd = HotkeyModifiers.None;
		private Keys _hotkeyKey                = Keys.None;
		private Keys _hotkeyKeyFade            = Keys.None;
		private Keys _hotkeyKeyInputWnd        = Keys.None;
	
		public
		Configuration(GDIPStrings gdi)
		{
			_gdi = gdi;
		}
		public void
		LoadConfigurations(String path)
		{
			Rectangle rectscreen = Screen.GetWorkingArea(_gdi);
			StringDictionary sd = Setting.GetSettings(path);

			_tonedownInterval = StringToInt32(sd["window.tonedowninterval"], 5000); // 10
			_lines = StringToInt32(sd["window.lines"], 10); // 10
			_shadow = (String.Compare(sd["window.shadow"], "true", true) == 0);
			_useFullFadeOut = (String.Compare(sd["window.fade.usefullfadeout"], "false", true) != 0);
			_revertEnter = (String.Compare(sd["window.inputform.revertenter"], "true", true) == 0);
			
			if (sd["window.fade.ratio"] != null) _FadeRatio = Byte.Parse(sd["window.fade.ratio"]);
			if (sd["window.fade.wait"] != null) _FadeWait = Double.Parse(sd["window.fade.wait"]);
			if (sd["window.opacity"] != null) _logFormOpacity = Byte.Parse(sd["window.opacity"]);
			if (sd["window.input.opacity"] != null) _inputFormOpacity = Byte.Parse(sd["window.input.opacity"]);
			if (sd["window.opacity.minimum"] != null)
				_logFormOpacityMin = Byte.Parse(sd["window.opacity.minimum"]);
			else
				_logFormOpacityMin = (Byte)(_logFormOpacity * 0.3);

			if (sd["window.keyword"] != null) _keyword = new Regex(sd["window.keyword"]);
			if (sd["window.browser"] != null) _browserPath = sd["window.browser"];


			// font
			String fontFamily = "Tahoma";
			Single fontSize = 9;
			FontStyle fontStyle = FontStyle.Regular;
			if (sd["window.font.family"] != null)
				fontFamily = sd["window.font.family"];
			if (sd["window.font.size"] != null)
				fontSize = Single.Parse(sd["window.font.size"]);
			if (String.Compare(sd["window.font.bold"], "true", true) == 0)
				fontStyle = FontStyle.Bold;
			if (String.Compare(sd["window.font.italic"], "true", true) == 0) {
				if (fontStyle != FontStyle.Regular) {
					fontStyle |= FontStyle.Italic;
				} else {
					fontStyle = FontStyle.Italic;
				}
			}
			_gdi.Font = _textFont = new System.Drawing.Font(
				  fontFamily
				, fontSize
				, fontStyle
			);
			_stringHeight = (Int32)_gdi.Graphics.MeasureString("|", _textFont).Height + 2;
			
			_width = StringToInt32(sd["window.width"], rectscreen.Width / 2);
			_left = StringToInt32(sd["window.left"], rectscreen.Left + _width);
			_top = StringToInt32(sd["window.top"], (rectscreen.Height - (_stringHeight * _lines)) - 2);
			
			_inputFormLeft = StringToInt32(sd["window.inputform.left"], _left);
			_inputFormTop = StringToInt32(sd["window.inputform.top"], _top);

			// Hotkey
			// スタイルが変わるとHandleが変わる。
			_hotkeyKeyInputWnd = _hotkeyKeyFade = _hotkeyKey = Keys.None;
			_hotkeyModInputWnd = _hotkeyModFade = _hotkeyMod = HotkeyModifiers.None;
			if (sd["window.visible.hotkey.modifiers"] != null && sd["window.visible.hotkey.key"] != null) {
				try {
					_hotkeyMod = (HotkeyModifiers)Enum.Parse(typeof(HotkeyModifiers), sd["window.visible.hotkey.modifiers"], true);
					_hotkeyKey = (Keys)Enum.Parse(typeof(Keys), sd["window.visible.hotkey.key"], true);
				} catch (System.ArgumentException) {
					// 不正
				}
			}
			if (sd["window.fade.hotkey.modifiers"] != null && sd["window.fade.hotkey.key"] != null) {
				try {
					_hotkeyModFade = (HotkeyModifiers)Enum.Parse(typeof(HotkeyModifiers), sd["window.fade.hotkey.modifiers"], true);
					_hotkeyKeyFade = (Keys)Enum.Parse(typeof(Keys), sd["window.fade.hotkey.key"], true);
				} catch (System.ArgumentException) {
					// 不正
				}
			}
			if (sd["window.inputform.hotkey.modifiers"] != null && sd["window.inputform.hotkey.key"] != null) {
				try {
					_hotkeyModInputWnd = (HotkeyModifiers)Enum.Parse(typeof(HotkeyModifiers), sd["window.inputform.hotkey.modifiers"], true);
					_hotkeyKeyInputWnd = (Keys)Enum.Parse(typeof(Keys), sd["window.inputform.hotkey.key"], true);
				} catch (System.ArgumentException) {
					// 不正
				}
			}
			
			// inputform
			if (sd["window.inputform.use"] != null) {
				_useInputForm = Boolean.Parse(sd["window.inputform.use"]);
			}
			// 以前のバージョンで使っていたもの
			if (sd["-.window.inputform.use"] != null) {
				_useInputForm = Boolean.Parse(sd["-.window.inputform.use"]);
			}
		}
		
		//
		// properties
		//
		[Category("ホットキー"), Description("同時に押される特殊なキーを指定します。")]
		public HotkeyModifiers
		LogWindowHotkeyModifier
		{
			get { return _hotkeyMod; }
			set { _hotkeyMod = value; }
		}
		[Category("ホットキー"), Description("同時に押される特殊なキーを指定します。")]
		public HotkeyModifiers
		InputWindowHotkeyModifier
		{
			get { return _hotkeyModInputWnd; }
			set { _hotkeyModInputWnd = value; }
		}
		[Category("ホットキー"), Description("同時に押される特殊なキーを指定します。")]
		public HotkeyModifiers
		LogWindowFadeHotkeyModifier
		{
			get { return _hotkeyModFade; }
			set { _hotkeyModFade = value; }
		}
			
		[Category("ホットキー"), Description("メニューを表示するためのキーを指定します。")]
		public Keys
		LogWindowHotkey
		{
			get { return _hotkeyKey; }
			set { _hotkeyKey = value; }
		}
		[Category("ホットキー"), Description("発言フォームを表示するためのキーを指定します。")]
		public Keys
		InputWindowHotkey
		{
			get { return _hotkeyKeyInputWnd; }
			set { _hotkeyKeyInputWnd = value; }
		}
		[Category("ホットキー"), Description("ログのフェード状態を切り替えるためのキーを指定します。")]
		public Keys
		LogWindowFadeHotkey
		{
			get { return _hotkeyKeyFade; }
			set { _hotkeyKeyFade = value; }
		}
		
		public Double
		FadeWait
		{
			get { return _FadeWait; }
			set { _FadeWait = value; }
		}
		public Byte
		FadeRatio
		{
			get { return _FadeRatio; }
			set { _FadeRatio = value; }
		}
		public Byte
		LogWindowOpacity
		{
			get { return _logFormOpacity; }
			set { _logFormOpacity = value; }
		}
		public Byte
		LogWindowOpacityMinimum
		{
			get { return _logFormOpacityMin; }
			set { _logFormOpacityMin = value; }
		}
		public Byte
		InputWindowOpacity
		{
			get { return _inputFormOpacity; }
			set { _inputFormOpacity = value; }
		}
		
		public Boolean
		Shadow
		{
			get { return _shadow; }
			set { _shadow = value; }
		}
		public Boolean
		UseInputForm
		{
			get { return _useInputForm; }
			set { _useInputForm = value; }
		}
		public Boolean
		useFullFadeOut
		{
			get { return _useFullFadeOut; }
			set { _useFullFadeOut = value; }
		}
		public Boolean
		revertEnter
		{
			get { return _revertEnter; }
			set { _revertEnter = value; }
		}
		public Int32
		TonedownInterval
		{
			get { return _tonedownInterval; }
			set { _tonedownInterval = value; }
		}
		public Int32
		StringHeight
		{
			get { return _stringHeight; }
			set { _stringHeight = value; }
		}
		public Int32
		lines
		{
			get { return _lines; }
			set { _lines = value; }
		}
		public Int32
		Top
		{
			get { return _top; }
			set { _top = value; }
		}
		public Int32
		Left
		{
			get { return _left; }
			set { _left = value; }
		}
		public Int32
		Width
		{
			get { return _width; }
			set { _width = value; }
		}
		public Int32
		InputFormTop
		{
			get { return _inputFormTop; }
			set { _inputFormTop = value; }
		}
		public Int32
		InputFormLeft
		{
			get { return _inputFormLeft; }
			set { _inputFormLeft = value; }
		}
		public String
		BrowserPath
		{
			get { return _browserPath; }
			set { _browserPath = value; }
		}
		public Regex
		Ignore
		{
			get { return _ignore; }
			set { _ignore = value; }
		}
		public Regex
		Keyword
		{
			get { return _keyword; }
			set { _keyword = value; }
		}
			
		//
		private static Int32
		StringToInt32(String s, Int32 dvalue)
		{
			try {
				return Int32.Parse(s);
			} catch (ArgumentNullException) {
				return dvalue;
			} catch (FormatException) {
				return dvalue;
			} // throw
		}
	}
}