/*
 * ConsoleForm.cs
 * $Id: ConsoleForm.cs,v 1.3 2003/11/24 10:09:35 Mayuki Sawatari Exp $
 *
 * Copyright (c) 2003 Mayuki Sawatari, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE MISUZILLA.ORG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Specialized;
using System.Runtime.InteropServices;

using Misuzilla.Net.GDIPlus.GDIPStrings;

namespace
Misuzilla.Net.GDIPlus.GDIPStrings
{
	internal class ConsoleForm : Form
	{
		public delegate void MessageSendEventHandler(String msg);
		public event MessageSendEventHandler MessageSend;

		private TextBox _textBox = null;
		private Int32 _lines = 10;
		private Boolean _ignoreEnter = false;
		private Boolean _revertEnter = false;

		//
		public
		ConsoleForm()
		{
			InitializeComponents();
		}
		
		private void
		InitializeComponents()
		{
			// form
			//this.BackColor = Color.White;
			this.ShowInTaskbar = false;
/**/		this.MinimizeBox = false;
			this.MaximizeBox = false;
			this.ControlBox = false;
			this.TopMost = true;
			//this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
			this.FormBorderStyle = FormBorderStyle.None;
			this.Text = "(Enterキーで描画/Shift+Enterで改行/最大" + _lines.ToString() + "行)";
			this.Font = new Font("Tahoma", 8);
			
			_textBox = new TextBox();
			_textBox.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Right;
			_textBox.AutoSize = false;
			_textBox.Multiline = true;
			_textBox.BorderStyle = BorderStyle.None;
			_textBox.KeyPress += new KeyPressEventHandler(TextBoxKeyPressed);
			_textBox.KeyDown += new KeyEventHandler(TextBoxKeyDown);
			_textBox.KeyUp += new KeyEventHandler(TextBoxKeyUp);

			this.Width = 400;
			this.Height = _textBox.PreferredHeight + 4 + this.Font.Height * _lines;
			_textBox.Bounds = new Rectangle(1, this.Font.Height+2, this.ClientSize.Width - 2, this.ClientSize.Height - (this.Font.Height + 4));
			_textBox.Update();
			this.Controls.Add(_textBox);
			
			this.ContextMenu = new ContextMenu();
			this.ContextMenu.MenuItems.Add(new MenuItem("最小化(&N)", new EventHandler(ContextMenuMinimizeClicked)));
			this.ContextMenu.MenuItems.Add(new MenuItem("-"));
			this.ContextMenu.MenuItems.Add(new MenuItem("キャンセル(&C)"));
		}

		protected override bool ProcessDialogKey(Keys k) {
		    switch(k){
		    	// TextBox上でCTRL+Aキーが押されたら全選択
				case Keys.A | Keys.Control:
					if(this.ActiveControl is TextBox){
						TextBox txt = (TextBox)this.ActiveControl;
						txt.SelectionStart = 0;
						txt.SelectionLength = txt.Text.Length;

						return true;
					}
					break;
			}
			return base.ProcessDialogKey(k);
		}

		private void
		ContextMenuMinimizeClicked(Object sender, EventArgs e)
		{
			this.Visible = false;
		}
		private void
		ContextMenuClicked(Object sender, EventArgs e)
		{
		}
		
		private void
		TextBoxKeyUp(Object sender, KeyEventArgs e)
		{
		}
		private void
		TextBoxKeyDown(Object sender, KeyEventArgs e)
		{
			// Enter キー入力時の処理
			if (e.KeyCode == Keys.Enter) {
				// revertEnter が false なら Shift キーが押されていないとき
				// revertEnter が true  なら Shift キーが押されているときに
				// 描画処理へ
				if ( (_revertEnter == false && e.Shift == false)
				  || (_revertEnter == true  && e.Shift == true) ) {
					if (this.MessageSend != null) {
						this.MessageSend(_textBox.Text);
						_ignoreEnter = true;
						e.Handled = true;
					}
				}
			}
			if (e.Control && (e.KeyCode == Keys.Space)) {
				// Ctrl + Space でメニュー
				this.ContextMenu.Show(this, new Point(0, 0));
				e.Handled = true;
			}
		}
		private void
		TextBoxKeyPressed(Object sender, KeyPressEventArgs e)
		{
			// Enter キーを押したときの処理
			if (e.KeyChar == (Char)13) {
				// キー無視フラグが立っていたら一回だけ無視する
				if (_ignoreEnter) {
					_ignoreEnter = false;
					e.Handled = true;
				}
			}
		}
		
		protected override void
		OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);
			// フォームドラッグで移動。
			if (e.Button == MouseButtons.Left) {
				Double op = this.Opacity;
				this.Opacity = 0.55;
				ReleaseCapture();
				// Int32 WM_NCLBUTTONDOWN = 0xa1;
				// Int32 HTCAPTION = 0x02;
				SendMessage(this.Handle, 0xa1, 0x02, IntPtr.Zero);
				this.Opacity = op;
			}
		}
		[DllImport("user32.dll")]
		private static extern Int32
		ReleaseCapture();
		
		[DllImport("user32.dll")]
		private static extern Int32
		SendMessage(IntPtr hWnd, Int32 wMsg, Int32 wParam, IntPtr lParam);

		protected override void
		OnActivated(EventArgs e)
		{
			base.OnActivated(e);
			this.Focus();
			this.Refresh();
			_textBox.Focus();
		}
		protected override void
		OnDeactivate(EventArgs e)
		{
			base.OnDeactivate(e);
			this.Refresh();
		}
		protected override void
		OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);
			
			Color borderColor, fontColor;
			if (Form.ActiveForm == this) {
				borderColor = Color.FromKnownColor(KnownColor.ActiveCaption);
				fontColor = Color.FromKnownColor(KnownColor.ActiveCaptionText);
			} else {
				borderColor = Color.FromKnownColor(KnownColor.InactiveCaption);
				fontColor = Color.FromKnownColor(KnownColor.InactiveCaptionText);
			}
			
			e.Graphics.DrawRectangle(new Pen(borderColor), new Rectangle(0, 0, this.ClientSize.Width - 1, this.ClientSize.Height - 1));
			e.Graphics.FillRectangle(new SolidBrush(borderColor), new Rectangle(2, 2, this.ClientSize.Width - 4, this.Font.Height+2));
			e.Graphics.DrawString(this.Text, this.Font, new SolidBrush(fontColor), 2F, 2F);
		}
		protected override void
		OnTextChanged(EventArgs e)
		{
			base.OnTextChanged(e);
			this.Refresh();
		}
		private void
		changeWindowTitle()
		{
			if (_revertEnter == true) {
				this.Text = "(Shift+Enterキーで描画/Enterで改行/最大" + _lines.ToString() + "行)";
			} else {
				this.Text = "(Enterキーで描画/Shift+Enterで改行/最大" + _lines.ToString() + "行)";
			}
		}
		
		//
		// properties
		//
		public Int32
		lines
		{
			get { return _lines; }
			set {
				if (lines > 0) {
					_lines = value;
					this.changeWindowTitle();
				}
				this.Height = _textBox.PreferredHeight + 4 + this.Font.Height * _lines;
			}
		}
		public Boolean
		revertEnter
		{
			get { return _revertEnter; }
			set {
				_revertEnter = value;
				this.changeWindowTitle();
			}
		}
	}
	class TextBoxWithBackText : TextBox
	{
		public String BackText = "";
		public  
		TextBoxWithBackText()
		{
			this.ResizeRedraw = true;
		}
		protected override void
		OnTextChanged(EventArgs e)
		{
			base.OnTextChanged(e);
			PaintBackgroundText();
		}
		private void
		PaintBackgroundText()
		{
			Console.WriteLine(BackText);
			Graphics g = this.CreateGraphics();//Graphics.FromHwnd(this.Handle);
			g.DrawString(BackText, this.Font, new SolidBrush(Color.Gray), this.Width - g.MeasureString(BackText, this.Font).Width, 0F);
			g.Dispose();
			g = null;
		}
	}
}
