/*
 * GDIPStrings.cs - modified from Ircv.cs
 * $Id: GDIPStrings.cs,v 1.01 2010/04/29 00:49 michirou Exp $
 *
 * Original copyright:
 * --------------------------------------------------------------------------
 * $Id: ircv.cs,v 1.19 2004/03/31 08:09:10 Mayuki Sawatari Exp $
 *
 * Copyright (C) 2003-2004 Mayuki Sawatari, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE MISUZILLA.ORG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * --------------------------------------------------------------------------
 */
 
using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;
using System.Drawing;
using System.Resources;
using System.Collections;
using System.Collections.Specialized;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

using Misuzilla.Utilities;
using Misuzilla.Windows.Forms;
using Misuzilla.Net.GDIPlus.GDIPStrings;

namespace
Misuzilla.Net.GDIPlus.GDIPStrings
{
	public class
	GDIPStrings : AlphaForm
	{
		// 
		private Configuration _conf = null;
		private Boolean _prevVisibleInputForm = false;
		private Boolean _tonedown = false;
		private System.Windows.Forms.Timer _timer;
		private NotifyIcon _notifyIcon;
		private Int32 _prevTime = 0;
		private String _confFileName = "default.xml";
		private String _oldMsg = "";
		
		private Bitmap _backbuff = null;
		private Graphics _backbuffgOver = null;
		private Graphics _backbuffgCopy = null;

		private readonly Brush _clearBrush = new SolidBrush(Color.FromArgb(0, 255, 255, 255));
		
		private MenuItem _uriLogMenu = null;
		private MenuItem _showInputFormMenu = null;
		private Regex _uriRegex = new Regex("(?:(?<uri>h?ttps?://[-_.!~*'()a-zA-Z0-9;/?:@&=+$,%#]+)|(?<text>(?:(?!h?ttps?://[-_.!~*'()a-zA-Z0-9;/?:@&=+$,%#]+).)+))");
		private Regex _timeformatRegex = new Regex("{time:([^}]*?)}");
		
		private Int32 _uTaskbarRestartMsg = 0;

		private ConsoleForm _inputForm = null;

		[STAThread]
		public static void
		Main(string[] args)
		{
			Application.Run(new GDIPStrings());
		}
		
		public
		GDIPStrings()
		{
		}
		
		protected override void
		OnLoad(EventArgs e)
		{
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(UnhandledExceptionCatched);
			String[] args = Environment.GetCommandLineArgs();
			this.Text = "GDIPStrings";
			this.ShowInTaskbar = false;
			this.TopMost = true;
			this.FormBorderStyle = FormBorderStyle.None;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.ControlBox = false;
			this.StartPosition = FormStartPosition.Manual;
			this.TransparentMouseMessage = true;
			this.Visible = true;

			ResourceManager rm = new ResourceManager("resource", System.Reflection.Assembly.GetExecutingAssembly());
			_notifyIcon = new NotifyIcon();
			_notifyIcon.Icon = (Icon)rm.GetObject("001");
			_notifyIcon.Text = "GDIPStrings";
			_notifyIcon.MouseUp += new MouseEventHandler(NotifyIconMouseUp);
			_notifyIcon.DoubleClick += new EventHandler(NotifyIconDoubleClicked);
			_notifyIcon.Visible = true;
			_notifyIcon.ContextMenu = new ContextMenu();
			_uriLogMenu = new MenuItem("アドレス リスト(&U)");
			_showInputFormMenu = new MenuItem("入力フォームを表示(&S)", new EventHandler(MenuInputFormVisibleClicked));
			_notifyIcon.ContextMenu.MenuItems.Add(_uriLogMenu);
			_notifyIcon.ContextMenu.MenuItems.Add(_showInputFormMenu);
			_notifyIcon.ContextMenu.MenuItems.Add(new MenuItem("-"));
			_notifyIcon.ContextMenu.MenuItems.Add(new MenuItem("GDIPStrings の終了(&X)", new EventHandler(MenuExitClicked)));

			_inputForm = new ConsoleForm();
			_inputForm.VisibleChanged += new EventHandler(InputFormVisibleChanged);

			// configure
			try {
				if (args.Length > 1)
					_confFileName = args[1];
				LoadSettings(_confFileName);
			} catch (System.Xml.XmlException xe) {
				ShowErrorMessage(xe.Message);
				this.Close();
				return;
			} catch (System.IO.IOException ie) {
				ShowErrorMessage(ie.Message);
				this.Close();
				return;
			}

			_timer = new System.Windows.Forms.Timer();
			_timer.Tick += new EventHandler(TimerOnTimed);
			_timer.Interval = 50;
			_timer.Start();
			
			_inputForm.Show();
			_inputForm.MessageSend += new ConsoleForm.MessageSendEventHandler(this.InputFormOnMessageSend);
			_inputForm.lines = _conf.lines;
			_inputForm.revertEnter = _conf.revertEnter;

			// fixed : 2010/04/29
			// useFullFadeOut が true の時は最初を完全透過にする
			if (_conf.useFullFadeOut == true) {
				this.Update(0);
			}

			base.OnLoad(e);
		}

		private void
		InputFormOnMessageSend(String msg)
		{
			// コマンド処理
			switch (msg.ToLower()) {
			case "/gdip:reload":
				try {
					LoadSettings(_confFileName);
				} catch (System.Xml.XmlException xe) {
					DrawStringWrapper("設定ファイルの再読み込みでエラーが発生しました。\n" + xe.Message);
				}
				return;
			default:
				DrawStringWrapper(msg);
				break;
			}
			_oldMsg = msg;
		}

		private void
		OpenUri(String uri)
		{
			if (uri.StartsWith("ttp"))
				uri = "h" + uri;
			
			if (_conf.BrowserPath != "" && File.Exists(_conf.BrowserPath)) {
				System.Diagnostics.Process.Start(_conf.BrowserPath, uri);
			} else {
				System.Diagnostics.Process.Start(uri);
			}
		}
		private void
		MenuURIItemClicked(Object sender, EventArgs e)
		{
			this.OpenUri(((EllipsisMenuItem)sender).FullText);
		}
		private void
		MenuInputFormVisibleClicked(Object sender, EventArgs e)
		{
			_inputForm.Visible = !_inputForm.Visible;
		}
		private void
		MenuExitClicked(Object sender, EventArgs e)
		{
			this.Close();
		}
		private void
		InputFormVisibleChanged(Object sender, EventArgs e)
		{
			_showInputFormMenu.Checked = _inputForm.Visible;
		}
		private void
		NotifyIconDoubleClicked(Object sender, EventArgs e)
		{
			if (_uriLogMenu.MenuItems.Count > 0) {
				this.OpenUri(((EllipsisMenuItem)_uriLogMenu.MenuItems[_uriLogMenu.MenuItems.Count - 1]).FullText);
			}
		}
		private void
		NotifyIconMouseUp(Object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Middle) this.NotifyIconDoubleClicked(sender, e);
			if (e.Button != MouseButtons.Left) return;
			this.Visible = !this.Visible;
			if (this.Visible) {
				_inputForm.TopMost = this.TopMost = false;
				_inputForm.TopMost = this.TopMost = true;
				this.Show();
			}
		}
		private void
		UnhandledExceptionCatched(Object sender, UnhandledExceptionEventArgs e)
		{
			foreach (String s in ((Exception)e.ExceptionObject).ToString().Split('\n')) {
				this.DrawStringWrapper("GDIPStrings :"+ s);
			}
			StreamWriter sw = new StreamWriter("GDIPStrings.stacktrace.txt", true, System.Text.Encoding.Default);
			sw.WriteLine("Date: {0}", DateTime.Now.ToString());
			sw.WriteLine("UnhandledException: ((Exception)e.ExceptionObject).ToString");
			sw.WriteLine("============================================================");
			sw.WriteLine(((Exception)e.ExceptionObject).ToString());
			sw.WriteLine("");
			sw.Close();
		}

		private void
		ShowErrorMessage(String str)
		{
			MessageBox.Show(str, "GDIPStrings", MessageBoxButtons.OK, MessageBoxIcon.Stop);
		}

		private void
		LoadSettings(String path)
		{
			Configuration conf = new Configuration(this);
			conf.LoadConfigurations(path);

			_inputForm.Opacity = (Double)conf.InputWindowOpacity / 255;

			this.Width = conf.Width;
			this.Height = (conf.StringHeight * (conf.lines + 1));
			this.Left = conf.Left;
			this.Top = conf.Top;
			
			_inputForm.Left = conf.InputFormLeft;
			_inputForm.Top = conf.InputFormTop;

			if (_backbuff != null) _backbuff.Dispose();
			if (_backbuffgOver != null) _backbuffgOver.Dispose();
			if (_backbuffgCopy != null) _backbuffgCopy.Dispose();
			_backbuff = new Bitmap(this.Width, this.Height);
			_backbuffgOver = Graphics.FromImage(_backbuff);
			_backbuffgCopy = Graphics.FromImage(_backbuff);
			_backbuffgOver.Clear(Color.FromArgb(0, 255, 255, 255));
			_backbuffgCopy.Clear(Color.FromArgb(0, 255, 255, 255));
			_backbuffgOver.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
			_backbuffgOver.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceOver;
			_backbuffgCopy.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceCopy;
			//this.Graphics.Clear(Color.FromArgb(0, 255, 255, 255));
			//this.Update();
			
			// Hotkey
			// スタイルが変わるとHandleが変わる可能性も無きにしも非ず。
			if (conf.LogWindowHotkey != Keys.None) {
				RegisterHotKey(this.Handle, 0, conf.LogWindowHotkeyModifier, conf.LogWindowHotkey);
			}
			if (conf.LogWindowFadeHotkey != Keys.None) {
				RegisterHotKey(this.Handle, 1, conf.LogWindowFadeHotkeyModifier, conf.LogWindowFadeHotkey);
			}
			if (conf.InputWindowHotkey != Keys.None) {
				RegisterHotKey(this.Handle, 2, conf.InputWindowHotkeyModifier, conf.InputWindowHotkey);
			}

			
			// 一時的なもの
			//_showInputFormMenu.Visible = (sd["-.window.inputform.use"] != null && Boolean.Parse(sd["-.window.inputform.use"]));
			_showInputFormMenu.Visible = conf.UseInputForm;

			this.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
			this.Graphics.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceCopy;

			_conf = conf;
		}

		protected override void
		OnClosing(System.ComponentModel.CancelEventArgs e)
		{
			base.OnClosing(e);
			UnregisterHotKey(this.Handle, 0);
			UnregisterHotKey(this.Handle, 1);
			UnregisterHotKey(this.Handle, 2);
			_notifyIcon.Visible = false;
		}
		public new Boolean
		Visible
		{
			get { return base.Visible; }
			set
			{
				base.Visible = value;
				if (_notifyIcon == null) return;
				ResourceManager rm = new ResourceManager("resource", System.Reflection.Assembly.GetExecutingAssembly());
				if (value) {
					_inputForm.Visible = _prevVisibleInputForm;
					_notifyIcon.Icon = (Icon)rm.GetObject("001");
					_notifyIcon.Text = "GDIPStrings";
				} else {
					_prevVisibleInputForm = _inputForm.Visible;
					_inputForm.Visible = false;
					_notifyIcon.Icon = (Icon)rm.GetObject("002");
					_notifyIcon.Text = "GDIPStrings[非表示]";
				}
			}
		}
		private void
		TimerOnTimed(Object sender, EventArgs e)
		{
			if (_prevTime == 0) return;
			if (_prevTime + _conf.TonedownInterval > Environment.TickCount) return;

			_prevTime = 0;
			if (_oldMsg.Length == 0 && _conf.useFullFadeOut == true) {
				// メッセージが空白で useFullFadeOut が true なら
				// ここで消去しておく
				this.Graphics.DrawImage(_backbuff, 0, 0);
			} else {
				// そうでなければフェードアウト
				this.FadeoutWindow(_conf.FadeWait, _conf.FadeRatio, (Byte)(_conf.LogWindowOpacityMinimum));
			}
			_tonedown = true;
		}
		
		protected override void
		OnHandleCreated(EventArgs e)
		{
			// タスクトレイのアイコン復活のためのメッセージ登録
			_uTaskbarRestartMsg = RegisterWindowMessage("TaskbarCreated");
			base.OnHandleCreated(e);
		}
		protected override void
		WndProc(ref Message m)
		{
			// タスクトレイのアイコン
			if (m.Msg == _uTaskbarRestartMsg && _uTaskbarRestartMsg != 0)
				_notifyIcon.Visible = true;
			
			// WM_HOTKEY
			if (m.Msg == 0x0312) {
				switch ((Int32)m.WParam) {
				case 0: // visible
					this.Visible = !this.Visible;
					break;
				case 1: // fade
					if (_prevTime == 0) {
						// 現在フェードアウトしてたらフェードイン
						if (_tonedown) {
							this.FadeinWindow(_conf.FadeWait, _conf.FadeRatio, _conf.LogWindowOpacity);
						} else {
							this.FadeoutWindow(_conf.FadeWait, _conf.FadeRatio, (Byte)(_conf.LogWindowOpacityMinimum));
						}
						_tonedown = !_tonedown;
					}
					break;
				case 2: // visible
					if (this.Visible && _showInputFormMenu.Visible && _showInputFormMenu.Enabled) {
						_inputForm.Visible = !_showInputFormMenu.Checked;
						if (_inputForm.Visible) _inputForm.Activate();
					}
					break;
				}
			}
			
			base.WndProc(ref m);
		}	
		protected override CreateParams
		CreateParams
		{
			get
			{
				CreateParams cp = base.CreateParams;
				cp.ExStyle |= 0x000080; /* WS_EX_TOOLWINDOW */
				return cp;
			}
		}
		
		[MethodImpl(MethodImplOptions.Synchronized)]
		private void
		DrawStringWrapper(String msg)
		{
			Color c = Color.White;

			// バックバッファクリア
			_backbuffgCopy.FillRectangle(_clearBrush, 0, 0, this.Width, this.Height);
			// メッセージが空ならDraw（つまり消去）して終了
			if (msg.Length == 0) {
				if (_conf.useFullFadeOut == true) {
					// もしフェードイン・アウトが有効ならフェードアウトする
					this.FadeoutWindow(_conf.FadeWait, _conf.FadeRatio, 0);
				} else {
					// そうでなければ即消去
					this.Graphics.DrawImage(_backbuff, 0, 0);
					this.FadeinWindow(_conf.FadeWait, _conf.FadeRatio, _conf.LogWindowOpacity);
				}
				_prevTime = Environment.TickCount;
				_tonedown = false;
				return;
			}

			// 行単位で区切って Draw する
			Int32 offsetY = 1;
			foreach (String msglines in msg.Split('\n')) {
				Int32 offsetX = 0;
				MatchCollection msgparts = _uriRegex.Matches(msglines);
				foreach (Match match in msgparts) {
					// 一行描画
					using (SolidBrush stringBrush = new SolidBrush((match.Groups["uri"].Value != "") ? Color.LightBlue : c)) {
						DrawStringWithShadow(_backbuffgOver, 
							match.Value,
							this.Font,
							stringBrush,
							new RectangleF(offsetX, _conf.StringHeight*(offsetY-1), this.Width - offsetX, _conf.StringHeight*(offsetY)+2),
							_conf.Shadow);
					}
					offsetX += _backbuffgOver.MeasureString(match.Value, this.Font).ToSize().Width;

					// メニュー
					if (match.Groups["uri"].Value.Length != 0) {
						EllipsisMenuItem mitem = new EllipsisMenuItem(match.Value, new EventHandler(MenuURIItemClicked));
						_uriLogMenu.Visible = false;
						_uriLogMenu.MenuItems.Add(mitem);
						if (_uriLogMenu.MenuItems.Count > 30)
							_uriLogMenu.MenuItems.RemoveAt(1);
						_uriLogMenu.Visible = true;
					}
				} // end-of-foreach (Match match...
				offsetY ++;
				// 行数超過
				if (offsetY > _conf.lines) {
					break;
				}
			} // end-of-foreach (Match matchline...

			this.Graphics.DrawImage(_backbuff, 0, 0);
			this.FadeinWindow(_conf.FadeWait, _conf.FadeRatio, _conf.LogWindowOpacity);
			_prevTime = Environment.TickCount;
			_tonedown = false;
		}
	
		private static void
		DrawStringWithShadow(Graphics g, String s, System.Drawing.Font f, SolidBrush b, RectangleF rectf, Boolean shadow)
		{
			SolidBrush sbframe = null;
			SolidBrush sbshadow = null;
			StringFormat sf = new StringFormat();
			sf.Trimming = StringTrimming.EllipsisCharacter;
			sf.FormatFlags = StringFormatFlags.NoWrap;

			Byte opDiff = (Byte)(255 - b.Color.A);
			if (opDiff < 80) {
				sbframe = new SolidBrush(Color.FromArgb(150 - opDiff, 0, 0, 0));
				sbshadow = new SolidBrush(Color.FromArgb(80 - opDiff, 0, 0, 0));
			}

			RectangleF rf1, rf2, rf3, rf4, rf5, rf6, rf7, rf8, rfshadow, rfstr;
			rf1 = rf2 = rf3 = rf4 = rf5 = rf6 = rf7 = rf8 = rfshadow = rfstr = rectf;
			rf2.Offset(0, 2);
			rf3.Offset(2, 0);
			rf4.Offset(2, 2);
			rf5.Offset(1, 0);
			rf6.Offset(1, 2);
			rf7.Offset(0, 1);
			rf8.Offset(2, 1);
			rfshadow.Offset(3, 3);
			rfstr.Offset(1, 1);

			// 枠
			if (sbframe != null) {
				g.DrawString(s, f, sbframe, rf1, sf);
				g.DrawString(s, f, sbframe, rf2, sf);
				g.DrawString(s, f, sbframe, rf3, sf);
				g.DrawString(s, f, sbframe, rf4, sf);
				g.DrawString(s, f, sbframe, rf5, sf);
				g.DrawString(s, f, sbframe, rf6, sf);
				g.DrawString(s, f, sbframe, rf7, sf);
				g.DrawString(s, f, sbframe, rf8, sf);
			}
			// ドロップシャドウ
			if (shadow && sbshadow != null)
				g.DrawString(s, f, sbshadow, rfshadow, sf);

			// 文字
			g.DrawString(s, f, b, rfstr, sf);
			
			if (sbshadow != null) sbshadow.Dispose();
			if (sbframe != null) sbframe.Dispose();
		}


		// MOD_ALT = 1 MOD_CTRL = 2 MOD_SHFT = 4
		[DllImport("user32.dll")]
		private static extern Int32
		RegisterHotKey(IntPtr hwnd, Int32 id, HotkeyModifiers fsModifiers, Keys vk);
		[DllImport("user32.dll")]
		private static extern Int32
		UnregisterHotKey(IntPtr hwnd, Int32 id);
		[DllImport("user32.Dll", CharSet=CharSet.Auto)]
		private static extern Int32
		RegisterWindowMessage([MarshalAs(UnmanagedType.LPTStr)] String lpString);

	}  
	[Flags]
	internal enum
	HotkeyModifiers
	{
		None  = 0,
		Alt   = 1,
		Ctrl  = 2,
		Shift = 4,
		Win   = 8,
	}

	internal class
	EllipsisMenuItem : MenuItem
	{
		public EllipsisMenuItem() : base() {}
		public EllipsisMenuItem(String s) : base(s) { this.Text = s; }
		public EllipsisMenuItem(String s, EventHandler e) : base(s, e) { this.Text = s; }
		
		public String _fullText = "";
		public new String
		Text
		{
			set
			{
				_fullText = value;
				if (base.Text.Length > 100)
					base.Text = value.Substring(0, 100) + "...";
				else
					base.Text = value;
			}
			get
			{
				return base.Text;
			}
		}
		public String
		FullText
		{
			get { return _fullText; }
			set { _fullText = value; }
		}
	}
}
